#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 15 23:05:34 2022

@author: wangh
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.stats.distributions import  t

def calculate_r2(obs,mod):
    residual = obs - mod
    ss_res = np.sum(residual**2)
    ss_tot = np.sum((obs - np.mean(obs))**2)
    r2 = 1-(ss_res/ss_tot)
    return r2,ss_res

def g93(Tinp):
    R= 8.314
    Ct1= 95000
    Ct2=230000
    Ts = 273.15+30
    T = Tinp+273.15
    Tm=314
    
    Ct = np.exp(Ct1*(T-Ts)/(R*Ts*T))/(1+np.exp(Ct2*(T-Tm)/(R*Ts*T)))
    return Ct

def g93_fit(Tinp,Ct1,Ct2,Tm):
    R= 8.314
    #Ct1= 95000
    #Ct2=230000
    Ts = 273.15+30
    T = Tinp+273.15
    #Tm=314
    
    Ct = np.exp(Ct1*(T-Ts)/(R*Ts*T))/(1+np.exp(Ct2*(T-Tm)/(R*Ts*T)))
    return Ct

def g99_fit(Tinp,Eopt,Ct1,Ct2,Tm):
    R= 0.008314

    #Ts = 273.15+30
    T = Tinp+273.15
    
    x = (1/Tm-1/T)/R
    #Tm=314
    
    g99 = (Eopt*Ct2*np.exp(Ct1*x))\
        /(Ct2 - Ct1*(1-np.exp(Ct2*x)))

    return g99

def Li_2023_Salix(T):
    y = 0.54*np.exp(0.13*T)
    return y

def Li_2023_Chamber(T):
    y = 0.003*np.exp(0.22*T)
    return y

def jing_fit(T):
    y = 17*np.exp(0.23*(T-20))
    return y

#======================================
colors = ["Red","royalblue",\
          "orchid","sienna",\
          "darkgreen","orchid",\
          "springgreen","salmon","red","grey","black"]
symbol = ["o","o","o","v",\
          "v","v","v","v","v","v","v"]

data_path = "./Data_input/"  
file_inp = data_path+"Fig1.Willow_Tem_Curve_2022-23.xlsx"


#==============Willow====================
print("=============Salix====================")
WL_Data = pd.read_excel(file_inp, sheet_name="Willow")
tem = np.array(WL_Data["Tem"].tolist())
norm_iso = np.array(WL_Data["Normlized"].tolist())
usage_flag = np.array(WL_Data["G99"].tolist())
willow_num = np.array(WL_Data["Number"].tolist())
willow_spc = np.array(WL_Data["Species"].tolist())

index = np.where(usage_flag == 1)
tem_wl = tem[index]
norm_iso_wl = norm_iso[index]
willow_num_fit = willow_num[index]
willow_spc_fit = willow_spc[index]
fig1, ax1 = plt.subplots(nrows=1, ncols=1, figsize=(8, 7 ))
wl_num_uniq = np.unique(willow_num)

p0_vars = [1.9,95,230,312.5]

x= np.arange(8,50,1)
spcs = ["Glauca","Pulchra","Reticulata"]
spcs_label = ["$\itSalix$ $\itglauca$",\
              "$\itSalix$ $\itpulchra$",\
              "$\itSalix$ $\itreticulata$"]

for i in range(len(wl_num_uniq)):
    inds = np.where(willow_num_fit == wl_num_uniq[i])
    tem_tmp = tem_wl[inds]
    iso_tmp = norm_iso_wl[inds]
    spc_tmp = willow_spc_fit[inds]
    print(spc_tmp[0])
    pars, pcov = curve_fit(g99_fit, tem_tmp, iso_tmp, p0=p0_vars)
    if spc_tmp[0] == "Glauca":
        color_plot = colors[0]
    elif spc_tmp[0] == "Pulchra":
        color_plot = colors[1]
    else:
        color_plot = colors[2]
    #print(iso_tmp)
    ax1.plot(tem_tmp,iso_tmp,color = color_plot,marker = "o",\
             markeredgecolor=color_plot, markeredgewidth=1.5,\
             linestyle="",markersize=10,alpha=0.5)
    mods = g99_fit(tem_tmp,pars[0],pars[1],pars[2],pars[3])
    ywl = g99_fit(x,pars[0],pars[1],pars[2],pars[3])
    r2,res_tot = calculate_r2(iso_tmp, mods)
    ax1.plot(x,ywl,color = color_plot,linestyle = "--",linewidth=4,alpha=0.3)

for i in range(len(spcs)):
    inds = np.where(willow_spc_fit == spcs[i])
    tem_tmp = tem_wl[inds]
    iso_tmp = norm_iso_wl[inds]
    spc_tmp = willow_spc_fit[inds]

    pars, pcov = curve_fit(g99_fit, tem_tmp, iso_tmp, p0=p0_vars)
    ywl = g99_fit(x,pars[0],pars[1],pars[2],pars[3])
    ax1.plot(x,ywl,color = colors[i],linestyle = "-",\
             linewidth=6,alpha=1.,label = spcs_label[i])

#===========================fit all lines====================
#p0_vars = [95000,230000,314]
p0_vars = [2.,95,230,313]
pars, pcov = curve_fit(g99_fit, tem_wl, norm_iso_wl, p0=p0_vars)
residual = norm_iso_wl - g99_fit(tem_wl,pars[0],pars[1],pars[2],pars[3])
ss_res = np.sum(residual**2)
ss_tot = np.sum((norm_iso_wl - np.mean(norm_iso_wl))**2)
r2_wl = 1-(ss_res/ss_tot)
  
pars_wl = pars

#uncertainties
N = len(norm_iso_wl)
N_p = 3
dof = N-N_p
alpha = 0.05 # 95% confidence interval = 100*(1-alpha)
tval = t.ppf(1.0-alpha/2., dof) 
p_max = pars #+ sigma_ab
p_min = pars #- sigma_ab
sigma = (np.diag(pcov))**0.5*tval
p_max = pars + sigma
p_min = pars - sigma  


ywl = g99_fit(x,pars[0],pars[1],pars[2],pars[3])
yg99 = g99_fit(x,p0_vars[0],p0_vars[1],p0_vars[2],p0_vars[3])   



 
ax1.plot(x,ywl,color = "darkorange",linestyle = "-",\
         linewidth=4,label = "All $\itSalix$ spp.")
ax1.plot(x,yg99,color = "darkgreen",linestyle="-",\
         linewidth=4,label="Default MEGAN")


ax1.set_xlabel("Temperature (°C)",fontsize =18)
ax1.spines.top.set_visible(False)
ax1.spines.right.set_visible(False)
ax1.set_ylabel("Normalized Isoprene Emission",fontsize=18)
ax1.tick_params(labelsize =18)    
plt.legend(fontsize=18)
plt.ylim([0,3])
ax1.legend(frameon = False,fontsize =12) 
plt.savefig("Figures/Ext.Fig3.Willow_G99_com.png",dpi=300,bbox_inches = 'tight')
plt.show()