#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 15 23:05:34 2022

@author: wangh
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from scipy.optimize import curve_fit
from datetime import datetime,timedelta
import scipy.stats
def get_period_data(time_ed,time_all,nhour,data_in):
    
    ind_bg = time_all.index(time_ed-timedelta(hours=nhour))
    ind_ed = time_all.index(time_ed)
    avg = np.nanmean(data_in[ind_bg:ind_ed])

    return avg
#get data
file_inp = "Data_input/TUAA2_Met_Jul_Aug2023_For_Ana.xlsx"
Met_Data = pd.read_excel(file_inp, sheet_name="Met_Data")
file_inp = "Data_input/Fig2.Willow_LongTerm_Isoprene_Emission.xlsx"
Iso_Data = pd.read_excel(file_inp, sheet_name="Longterm_Isoprene")
file_out = "Data_input/Ext.FigS5.Met_And_Isop_Data_Statistic.xlsx"
#------------------Met-----------------------
time_met = Met_Data["Time"].tolist()
#time_met = Met_Data["Time_Met"].tolist()
TEM  = np.array(Met_Data["Tem (C)"].tolist())
PPFD = np.array(Met_Data["PPFD (umol m-2 s-1)"].tolist())
RH   = np.array(Met_Data["RH (%)"].tolist())
WSPD = np.array(Met_Data["Windspeed"].tolist())
VPD  = np.array(Met_Data["VPD (kPa)"].tolist())


Time_met_all=[]
for i in time_met:
    time_tmp = datetime.strptime(i,'%m/%d/%Y %H:%M AKDT')
    Time_met_all.append(time_tmp)

willow_num  = np.array(Iso_Data["Tree#Leaf#"].tolist())
iso_wl = np.array(Iso_Data["Flux (nmol m-2 s-1) at 30C"].tolist())#/12.25
Time = np.array(Iso_Data["Time_Met"].tolist())
Date = np.array(Iso_Data["Date"].tolist())





iso_norm = np.zeros(len(iso_wl))
wl_num_uniq = np.unique(willow_num)
for i in range(len(wl_num_uniq)):
    print(wl_num_uniq[i])
    inds = np.where(willow_num == wl_num_uniq[i])
    iso_tmp_all = iso_wl[inds]
    ef = np.mean(iso_tmp_all)
    iso_norm[inds] = iso_tmp_all/ef

Tem_PDay = []
PPFD_PDay = []
VPD_PDay = []
for t in Date:
    time_tmp = t
    
    #3h
    Tem_tmp = get_period_data(time_tmp, Time_met_all, 48, TEM)
    PPFD_tmp = get_period_data(time_tmp, Time_met_all, 48, PPFD)
    VPD_tmp = get_period_data(time_tmp, Time_met_all, 48, VPD)
    Tem_PDay.append(Tem_tmp)
    PPFD_PDay.append(PPFD_tmp)
    VPD_PDay.append(VPD_tmp)

tem_int = np.concatenate((np.arange(1,72,1),np.array([120,168,240])))
Tem_R = np.zeros(len(tem_int))
PPFD_R = np.zeros(len(tem_int))
VPD_R = np.zeros(len(tem_int))
TP_R = np.zeros(len(tem_int))
Tem_P = np.zeros(len(tem_int))
PPFD_P = np.zeros(len(tem_int))
VPD_P = np.zeros(len(tem_int))
TP_P = np.zeros(len(tem_int))

for i in range(len(tem_int)):
    PPFD_all = []
    VPD_all = []
    Tem_all = []
    TP_all = []
    print("Tem =",tem_int[i])
    for t in Time: 
        time_tmp = t-timedelta(hours=0.5)
        Tem_tmp = get_period_data(time_tmp, Time_met_all, int(tem_int[i]), TEM)
        PPFD_tmp = get_period_data(time_tmp, Time_met_all, int(tem_int[i]), PPFD)
        VPD_tmp = get_period_data(time_tmp, Time_met_all, int(tem_int[i]), VPD)
        Tem_all.append(Tem_tmp)
        PPFD_all.append(PPFD_tmp)
        VPD_all.append(VPD_tmp)
        TP_all.append(Tem_tmp*PPFD_tmp)
    p = scipy.stats.pearsonr(np.log(iso_norm), Tem_all)
    Tem_R[i] = p[0]
    Tem_P[i] = p[1]
    p = scipy.stats.pearsonr(np.log(iso_norm), PPFD_all)
    PPFD_R[i] = p[0]
    PPFD_P[i] = p[1]
    p = scipy.stats.pearsonr(np.log(iso_norm), VPD_all)
    VPD_R[i] = p[0]
    VPD_P[i] = p[1]
    p = scipy.stats.pearsonr(np.log(iso_norm), TP_all)
    TP_R[i] = p[0]
    TP_P[i] = p[1]




data_out = pd.DataFrame(np.zeros((len(tem_int),9)),\
           columns = ["Time (hr)","Tem_r","Tem_p",\
                      "PPFD_r","PPFD_p","VPD_r","VPD_p","TP_r","TP_p"])

data_out.iloc[:,0]=tem_int
data_out.iloc[:,1]=Tem_R
data_out.iloc[:,2]=Tem_P
data_out.iloc[:,3]=PPFD_R
data_out.iloc[:,4]=PPFD_P
data_out.iloc[:,5]=VPD_R
data_out.iloc[:,6]=VPD_P
data_out.iloc[:,7]=TP_R
data_out.iloc[:,8]=TP_P
    
    
data_out.to_excel(file_out,sheet_name="Statistic")
fig1, ax1 = plt.subplots(nrows=1, ncols=1, figsize=(8, 7 ))
#ax1.plot(x,R2_all)
ax1.plot(tem_int,Tem_R,"-*",color = "red",label = "Tem.")
ax1.plot(tem_int,PPFD_R,"-*",color = "dodgerblue",label = "PPFD")
ax1.plot(tem_int,VPD_R,"-*",color = "darkorange",label = "VPD")
ax1.plot(tem_int,TP_R,"-*",color = "darkgreen",label = "Tem. x PPFD")
ax1.spines.top.set_visible(False)
ax1.spines.right.set_visible(False)
ax1.tick_params(axis='both', which='major', labelsize=18)
ax1.set_xlabel("Hours before the measurement",fontsize =20)    
ax1.set_ylabel("Correlation coefficient",fontsize=20)
ax1.legend()

#plt.xticks(x, vars_label)
#ax1.set_ylim([0,3])
ax1.set_ylim([0,1])
ax1.legend(fontsize= 20,frameon = False)
plt.savefig("Figures/Ext.Fig5.Willow_Longterm_Exp_Correlation.png",dpi=300,bbox_inches = 'tight')
plt.show()
