#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 15 23:05:34 2022

@author: wangh
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from scipy.optimize import curve_fit
from scipy.stats.distributions import  t

def calculate_r2(obs,mod):
    residual = obs - mod
    ss_res = np.sum(residual**2)
    ss_tot = np.sum((obs - np.mean(obs))**2)
    r2 = 1-(ss_res/ss_tot)
    return r2,ss_res 

def g99_fit(Tinp,Eopt,Ct1,Ct2,Tm):
    R= 0.008314

    #Ts = 273.15+30
    T = Tinp+273.15
    
    x = (1/Tm-1/T)/R
    #Tm=314
    
    g99 = (Eopt*Ct2*np.exp(Ct1*x))\
        /(Ct2 - Ct1*(1-np.exp(Ct2*x)))

    return g99

def Li_2023_Salix(T):
    y = 0.54*np.exp(0.13*T)
    return y

def Li_2023_Chamber(T):
    y = 0.003*np.exp(0.22*T)
    return y

def jing_fit(T):
    y = 17*np.exp(0.23*(T-20))
    return y
def Seco_2020(T):
    y = np.exp(np.log(14.5)/10*T)
    return y

def Lejish_2021(T):
    y = np.exp(0.23*(T-30))
    return y


def exp_fit_mg(Tinp,p1,p2):
    Tout = p1*np.exp((p2*(Tinp-30)))
    return Tout

def cal_ci(x,xi,yi,ss_res,pars,pcov):
    #calculate the 95 intervals
    N = len(xi)
    N_p = len(pars)
    dof = N-N_p
    alpha = 0.05 # 95% confidence interval = 100*(1-alpha)
    tval = t.ppf(1.0-alpha/2., dof) 
    #sigma = (np.diag(pcov))**0.5*tval
    RMSE = np.sqrt(ss_res/dof)
    ci = tval * RMSE * np.sqrt(1 / N + (x - np.mean(xi))**2 / np.sum((xi- np.mean(xi))**2))
    return ci
#======================================
colors = ["darkorange","royalblue",\
          "cyan","sienna",\
          "darkgreen","orchid",\
          "springgreen","salmon","red","grey"]
symbol = ["v","v","v","v",\
          "v","v","v","v","v","v","v"]

data_path = "./Data_input/"  
file_inp = data_path+"Fig1.Willow_Tem_Curve_2022-23.xlsx"


#==============Willow====================
print("=============Salix====================")
WL_Data = pd.read_excel(file_inp, sheet_name="Willow")
tem = np.array(WL_Data["Tem"].tolist())
norm_iso = np.array(WL_Data["Normlized"].tolist())
iso_flux = np.array(WL_Data["Isoprene (nmol m-2 s-1)"].tolist())
iso_flux_dry_mass = np.array(WL_Data["Isoprene per dry mass (ug g h-1)"].tolist())
usage_flag = np.array(WL_Data["G99"].tolist())
willow_num = np.array(WL_Data["Number"].tolist())
willow_spc = np.array(WL_Data["Species"].tolist())

index = np.where(usage_flag == 1)
tem_wl = tem[index]
norm_iso_wl = norm_iso[index]
willow_num_fit = willow_num[index]
fig1, [ax1,ax2] = plt.subplots(nrows=1, ncols=2, figsize=(16, 7 ))
wl_num_uniq = np.unique(willow_num)
p0_vars = [2.,95,230,313]

x= np.arange(8,50,1)


for i in range(len(wl_num_uniq)):
    inds = np.where(willow_num_fit == wl_num_uniq[i])
    tem_tmp = tem_wl[inds]
    iso_tmp = norm_iso_wl[inds]
    pars, pcov = curve_fit(g99_fit, tem_tmp, iso_tmp, p0=p0_vars)
    
    #print(iso_tmp)
    ax1.plot(tem_tmp,iso_tmp,color = colors[0],marker = "o",\
             markeredgecolor=colors[0], markeredgewidth=1.5,\
             linestyle="",markersize=10,alpha=0.3)


# fit the willow line
p0_vars = [2.,95,230,313]
pars, pcov = curve_fit(g99_fit, tem_wl, norm_iso_wl, p0=p0_vars)
residual = norm_iso_wl - g99_fit(tem_wl,pars[0],pars[1],pars[2],pars[3])
ss_res = np.sum(residual**2)
ss_tot = np.sum((norm_iso_wl - np.mean(norm_iso_wl))**2)
r2_wl = 1-(ss_res/ss_tot)

ci = cal_ci(x,tem_wl,norm_iso_wl,ss_tot,pars,pcov)
print("Parameters for Salix spp:",pars)

#uncertainties
N = len(norm_iso_wl)
N_p = 3
dof = N-N_p
alpha = 0.05 # 95% confidence interval = 100*(1-alpha)
tval = t.ppf(1.0-alpha/2., dof) 
p_max = pars #+ sigma_ab
p_min = pars #- sigma_ab
sigma = (np.diag(pcov))**0.5*tval
print("Parameters uncertaities: ", sigma)
p_max = pars + sigma
p_min = pars - sigma  


ywl = g99_fit(x,pars[0],pars[1],pars[2],pars[3])
y_salix_Li= Li_2023_Salix(x)/Li_2023_Salix(30)
y_chamber_Li= Li_2023_Chamber(x)/Li_2023_Chamber(30)
yjing= jing_fit(x)/jing_fit(30)
pars_Finse = [1,150.39,711.43,303.06]
yg99_Finse = g99_fit(x,pars_Finse[0],pars_Finse[1],\
                     pars_Finse[2],pars_Finse[3])
yg99 = g99_fit(x,p0_vars[0],p0_vars[1],p0_vars[2],p0_vars[3])   
pars_Abisko = [2,230.88,231.96,322.12]
yg99_Abisko = g99_fit(x,pars_Abisko[0],pars_Abisko[1],\
                     pars_Abisko[2],pars_Abisko[3])

#plot seco_2020
seco_2020 =Seco_2020(x)/Seco_2020(30)

#Lejish
lejish_2022 = Lejish_2021(x)    


x_0 = np.arange(10,35,1)
p0_vars = [2.,0.3]
yg99_Abisk_fit = g99_fit(x_0,pars_Abisko[0],pars_Abisko[1],\
                     pars_Abisko[2],pars_Abisko[3])
pars, pcov = curve_fit(exp_fit_mg,x_0, yg99_Abisk_fit, p0=p0_vars)
print("Seco et al.(2022):",pars[1])
#========================================================================
print("=====================Emission Factor================")
spc_uniq = np.unique(willow_spc)

x_ef1 = [0,3.5,6.5]
x_ef2 = [1.5,4.5,7.5]
y1 = np.zeros(3)
yerr1 = np.zeros(3)
y2 = np.zeros(3)
yerr2 = np.zeros(3)


for i in range(len(spc_uniq)):
    print(spc_uniq[i])
    inds = np.where(willow_spc == spc_uniq[i])
    tem_tmp = tem[inds]
    iso_tmp = iso_flux[inds]
    iso_dm_tmp = iso_flux_dry_mass[inds]
    
    #get 30 C
    inds = np.where((tem_tmp > 29) & (tem_tmp <31))
    iso_ef = iso_tmp[inds]
    
    iso_dm_ef = iso_dm_tmp[inds]
    
    print("EF per area:",np.mean(iso_ef))
    print("STD per area:",np.std(iso_ef))
    print("EF per dry mass:",np.mean(iso_dm_ef))
    print("STD per dry mass:",np.std(iso_dm_ef))
    yerr1[i]=np.std(iso_ef)
    y1[i]=np.mean(iso_ef)
    yerr2[i]=np.std(iso_dm_ef)
    y2[i]=np.mean(iso_dm_ef)
    #rint(iso_ef)

    
 
   
line1, = ax1.plot(x,yg99_Abisko,color = "turquoise",linestyle = "dashdot",\
         linewidth=2,label = "Seco et al. 2022 (Abisko-Stordalen, EC)")
line2, = ax1.plot(x,yg99_Finse,color = "Navy",linestyle = "dashdot",\
         linewidth=2,label = "Seco et al. 2022 (Finse, EC)")
line9, = ax1.plot(x,seco_2020,color = "darkred",linestyle = "dashdot",\
         linewidth=2,label = "Seco et al. 2020 (Abisko-Stordalen, EC)")
line3,  = ax1.plot(x,lejish_2022,color = "palegreen",\
         linestyle = "dashdot",linewidth=2,label = "Vettikkat et al. 2022 (Siikaneva, EC)")
line4, = ax1.plot(x,yjing,color = "violet",linestyle = "-",\
         linewidth=4,label = "Tang et al., 2016 (Abisko, GC)",alpha=0.9)
line5, = ax1.plot(x,y_chamber_Li,color = "deepskyblue",linestyle = "-",\
         linewidth=4,label = "Li et al. 2023 (Abisko, GC)",alpha=0.7)
line6, = ax1.plot(x,ywl,color = "darkorange",linestyle = "-",\
         linewidth=5,label = "$\itSalix$ spp. (Toolik, R${^2}$ = "+"${0:2.2f}$".format(r2_wl)+")",alpha = 0.8)
line7, = ax1.plot(x,yg99,color = "darkgreen",linestyle="-",\
         linewidth=5,label="MEGAN model", alpha = .7)
line8, = ax1.plot(x,y_salix_Li,color = "black",linestyle = "-",\
         linewidth=4,label = "Li et al. 2023 (Abisko, BC)")

ax1.fill_between(
        x, ywl-ci, ywl+ci, facecolor='lightpink', zorder=0, alpha=0.4
    )


#add legend to plot
ax1.legend(handles=[line6,line7,line8,line1,line2,line9,line3,line4,line5],frameon = False,fontsize =11.3)
ax1.tick_params(labelsize =20)    
ax1.set_xlabel("Temperature (°C)",fontsize = 22)
ax1.spines.top.set_visible(False)
ax1.spines.right.set_visible(False)
ax1.set_ylabel("Normalized Isoprene Emission",fontsize=22)
ax1.set_ylim([0,6])
ax3 = ax2.twinx()
#===================Emission Factors====================
ax2.errorbar(x_ef1,y1,yerr=yerr1,\
             ecolor="darkorange",elinewidth=5,\
             fmt ='o',markersize=15,\
             mec="Darkorange",mfc="White",markeredgewidth=5, \
             capsize=10,capthick=5)
    

ax3.errorbar(x_ef2,y2,yerr=yerr2,\
             ecolor="Darkgreen",elinewidth=5,\
             fmt ='o',markersize=15,\
             mec="Darkgreen",mfc="White",markeredgewidth=5, \
             capsize=10,capthick=5)

ax2.set_xlim([-1,8.5])
ax2.set_ylim([-5,35])
ax3.set_ylim([-1,70])
ax2.set_ylabel("Isoprene emission (nmol $m^{-2}$ $s^{-1}$)",\
               fontsize=20, color = "Darkorange")
ax3.set_ylabel("Isoprene emission (μg $g^{-1}$ $h^{-1}$)",\
               fontsize=20, color = "Darkgreen")


ax2.xaxis.tick_bottom()
ax2.tick_params(labelsize =20)
ax3.tick_params(labelsize =20)
ax2.set_xticks([0.75,3.75,6.75])
ax2.set_xticklabels(['$\itSalix$\n $\itglauca$','$\itSalix$\n $\itpulchra$',\
                     '$\itSalix$\n $\itreticulata$'])
ax2.spines.top.set_visible(False)
ax2.spines.right.set_visible(False)
ax3.spines.top.set_visible(False)
fig1.text(0.12,.92,"(a)",fontsize=22,color = "k",fontweight='bold')
fig1.text(0.55,.92,"(b)",fontsize=22,color = "k",fontweight='bold')

plt.savefig("Figures/Fig1.Willow_G99_com.png",dpi=300,bbox_inches = 'tight')
plt.show()