#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 15 23:05:34 2022

@author: wangh
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from scipy.optimize import curve_fit
from scipy.stats.distributions import  t

from datetime import datetime,timedelta

def calculate_r2(obs,mod):
    residual = obs - mod
    ss_res = np.sum(residual**2)
    ss_tot = np.sum((obs - np.mean(obs))**2)
    r2 = 1-(ss_res/ss_tot)
    return r2,ss_res




def get_period_data(time_ed,time_all,nday,data_in):
    
    ind_bg = time_all.index(time_ed)
    ind_ed = time_all.index(time_ed+timedelta(days=nday))
    #print(time_all[ind_bg],"-",time_all[ind_ed])
    #print(len(data_in[ind_bg:ind_ed]))
    avg = np.nanmean(data_in[ind_bg:ind_ed])
    std = np.nanstd(data_in[ind_bg:ind_ed])
    return avg,std
def exp_fit(Tinp,p1,p2):
    Tout = p1*np.exp((p2*(Tinp-24)))
    return Tout


def cal_ci(x,xi,yi,ss_res,pars,pcov):
    #calculate the 95 intervals
    N = len(xi)
    N_p = len(pars)
    dof = N-N_p
    alpha = 0.05 # 95% confidence interval = 100*(1-alpha)
    tval = t.ppf(1.0-alpha/2., dof) 
    #sigma = (np.diag(pcov))**0.5*tval
    RMSE = np.sqrt(ss_res/dof)
    ci = tval * RMSE * np.sqrt(1 / N + (x - np.mean(xi))**2 / np.sum((xi- np.mean(xi))**2))
    return ci
#======================================
colors = ["violet","royalblue",\
          "cyan","sienna",\
          "dodgerblue","salmon",\
          "springgreen","orchid"]
symbol = ["o","o","o","v",\
          "v","v","v"]
#======================================
print("=============Salix====================")
data_path = "./Data_input/" 
file_inp = data_path+"Fig2.Willow_LongTerm_Isoprene_Emission.xlsx"
WL_Data = pd.read_excel(file_inp, sheet_name="Longterm_Isoprene")
willow_num = np.array(WL_Data["Tree#Leaf#"].tolist())
iso_wl = (np.array(WL_Data["Flux (nmol m-2 s-1) at 30C"].tolist()))#/12.25
Time_obs = np.array(WL_Data["Date"].tolist())
x = np.arange(5,20,1)
print("EF:",np.nanmean(iso_wl)) 



#------------------Met-----------------------
file_inp = "Data_input/TUAA2_Met_Jul_Aug2023_For_Ana.xlsx"
Met_Data = pd.read_excel(file_inp, sheet_name="Met_Data")
time_met = Met_Data["Time"].tolist()
TEM  = np.array(Met_Data["Tem (C)"].tolist())
PPFD = np.array(Met_Data["PPFD (umol m-2 s-1)"].tolist())
RH   = np.array(Met_Data["RH (%)"].tolist())
WSPD = np.array(Met_Data["Windspeed"].tolist())
VPD  = np.array(Met_Data["VPD (kPa)"].tolist())

Time_met_all=[]
for i in time_met:
    time_tmp = datetime.strptime(i,'%m/%d/%Y %H:%M AKDT')
    Time_met_all.append(time_tmp)

    
date_st = datetime.strptime("07/16/2023 00:00",'%m/%d/%Y %H:%M')
date_ct = date_st
nday = 22

tem_avg =[]
tem_std =[]
tem_time = []
tem_min = []
tem_max = []
#get the average temperature
for i in range(nday):
    
    tem_avg_tmp,tem_std_tmp = get_period_data(date_ct, Time_met_all, 1 , TEM)
    tem_avg.append(tem_avg_tmp)
    tem_std.append(tem_std_tmp)
    tem_time.append(date_ct)
    tem_min.append(tem_avg_tmp-tem_std_tmp)
    tem_max.append(tem_avg_tmp+tem_std_tmp)
    if tem_avg_tmp > 16.5:
        print(date_ct,tem_avg_tmp)
    date_ct = date_ct + timedelta(hours=24)


#get the average temperature of preivous day
tem_1day_off=[]
for i in Time_obs:
    date_ct = i - timedelta(days=1)
    tem_avg_tmp,tem_std_tmp = get_period_data(date_ct, Time_met_all, 1 , TEM)
    tem_1day_off.append(tem_avg_tmp)

tem_1day_off = np.array(tem_1day_off)
tem_95per = np.zeros(len(tem_avg))
tem_jul_avg = np.zeros(len(tem_avg))
tem_95per[:]=16.5
tem_jul_avg[:]= 10.4


#==================================================================

wl_num_uniq = np.unique(willow_num)
'''
fig1 = plt.figure(tight_layout=True,figsize=(16, 16 ))
gs = gridspec.GridSpec(2, 2,hspace=0.3,wspace=0.3)

ax1 = fig1.add_subplot(gs[0, :])
ax2 = ax1.twinx()  # instantiate a second axes that shares the same x-axis
ax3 = fig1.add_subplot(gs[1,0])
'''
fig1 = plt.figure(tight_layout=True,figsize=(20, 6.8 ))
gs = gridspec.GridSpec(1, 14,hspace=0.3,wspace=2.2)

ax1 = fig1.add_subplot(gs[0, :8])
ax2 = ax1.twinx()  # instantiate a second axes that shares the same x-axis
ax3 = fig1.add_subplot(gs[0,9:])

iso_norm = np.zeros(len(iso_wl))
for i in range(len(wl_num_uniq)):
    print(wl_num_uniq[i])
    inds = np.where(willow_num == wl_num_uniq[i])
    time_tmp = Time_obs[inds]
    tem_1day_tmp = tem_1day_off[inds]
    iso_tmp = iso_wl[inds]
    iso_norm_tmp = iso_tmp/np.mean(iso_tmp)
    iso_norm[inds] = iso_norm_tmp
    
    
    ax3.plot(tem_1day_tmp,iso_norm_tmp,\
             color = colors[i],marker = symbol[i],\
             markeredgecolor=colors[i], markeredgewidth=4,\
             linestyle="",markersize=15,alpha=0.5)
    #(iso_tmp-np.mean(iso_tmp))/np.std(iso_tmp)
    ax1.plot(time_tmp,iso_norm_tmp,\
             color = colors[i],marker = symbol[i],\
             markeredgecolor=colors[i], markeredgewidth=4,\
             linestyle="",markersize=15,alpha=0.5,label=wl_num_uniq[i])

p0_vars = [2,0.3]
pars, pcov = curve_fit(exp_fit, tem_1day_off, iso_norm, p0=p0_vars)
mods = exp_fit(tem_1day_off,pars[0],pars[1])
r2,res_tot = calculate_r2(iso_norm, mods)

ci = cal_ci(x,tem_1day_off,iso_norm,res_tot,pars,pcov)

ysg_wl_exp = exp_fit(x,pars[0],pars[1])
ysg_megan_exp = exp_fit(x,2,0.05)

eq_txt = "y="+"{0:0.1f}".format(pars[0])+"$\ite$$^{0.22(T-24)}$"

        #eq_txt = "y={0:2.2f}${e^{1:.2f}}$".format(pars[0],abs(pars[1]))
r2_txt = "$\itR$$^2$={0:2.2f}".format(r2)


ax1.set_xlabel("Date",fontsize =20)
ax1.set_ylabel("Normalized isoprene emission",fontsize=20)
ax1.tick_params(axis='both', which='major', labelsize=15)
ax1.spines.top.set_visible(False)
ax2.plot(tem_time[3:], tem_95per[3:], color="red",linestyle='--',linewidth=4,alpha=0.5)
ax2.plot(tem_time, tem_jul_avg, color="royalblue",linestyle='--',linewidth=4,alpha=0.5)

ax2.set_ylabel('Daily Temp. (°C)', color="darkorange",fontsize=20)  # we already handled the x-label with ax1
ax2.plot(tem_time, tem_avg, color="darkorange",linestyle='-',linewidth=4)
ax2.fill_between(tem_time,tem_min, tem_max, \
                 facecolor='lightpink', zorder=0,alpha =0.3)   

ax2.tick_params(axis='both', which='major', labelsize=18)
ax2.spines.top.set_visible(False)
ax1.legend(frameon = False,loc = "upper left",fontsize=15)


ax3.plot(x,ysg_wl_exp,color = "darkorange",linestyle = "-",linewidth=6,\
         label="$\itSalix$ spp.") 
    
ax3.plot(x,ysg_megan_exp,color = "darkgreen",linestyle = "--",linewidth=6,\
         label="Default MEGAN") 

ax3.fill_between(
    x, ysg_wl_exp-ci, ysg_wl_exp+ci, facecolor='lightpink', zorder=0, alpha=0.4
)

ax3.text(10, 1.7, eq_txt, ha='center',fontsize =24,color = "darkorange")
ax3.text(8.7, 1.4, r2_txt, ha='center',fontsize =24,color = "darkorange")
ax3.tick_params(axis='both', which='major', labelsize=18)
ax3.set_xlabel("Previous Day Averaged Temp. (°C)",fontsize =20)
ax3.set_ylabel("Normalized isoprene emission",fontsize=20)
ax3.legend(fontsize= 20,frameon = False)
ax3.spines.top.set_visible(False)
ax3.spines.right.set_visible(False)

fig1.text(0.126,.895,"(a)",fontsize=22,color = "k",fontweight='bold')
fig1.text(0.649,.895,"(b)",fontsize=22,color = "k",fontweight='bold')
#fig1.text(0.557,.47,"(c)",fontsize=22,color = "k",fontweight='bold')

plt.savefig("Figures/Fig2.Willow_Longterm_Exp.png",dpi=300,bbox_inches = 'tight')
#ax1.legend(fontsize = 15)
