#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 15 23:05:34 2022

@author: wangh
"""

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
from datetime import datetime
from scipy.optimize import curve_fit

def calculate_r2(obs,mod):
    residual = obs - mod
    ss_res = np.sum(residual**2)
    ss_tot = np.sum((obs - np.mean(obs))**2)
    r2 = 1-(ss_res/ss_tot)
    return r2,ss_res

def exp_fit_mg(Tinp,p1,p2):
    Tout = p1*np.exp((p2*(Tinp-30)))
    return Tout
#======================================
colors = ["violet","royalblue",\
          "cyan","sienna",\
          "dodgerblue","salmon",\
          "springgreen","orchid"]
symbol = ["o","o","o","v",\
          "v","v","v"]



#===============MEGAN================
#------------------Met-----------------------
file_inp = "Data_input/Salix_Jul_Aug2023_Isop_MEGAN.xlsx"
MEGAN_Data = pd.read_excel(file_inp)
time_megan = MEGAN_Data["Time"].tolist()
#TEM  = np.array(MEGAN_Data["Tem (C)"].tolist())
Tem_MEGAN_all  = np.array(MEGAN_Data["Air Tem (C)"].tolist())
Leaf_Tem_MEGAN_all  = np.array(MEGAN_Data["Leaf Tem (C)"].tolist())
Default_MEGAN_all = np.array(MEGAN_Data["Default Isop (nmol m-2 s-1)"].tolist())
New_MEGAN_all     = np.array(MEGAN_Data["New Isop (nmol m-2 s-1)"].tolist())
time_mg_bg = datetime.strptime("07/16/2023 00:00",'%m/%d/%Y %H:%M')
time_mg_ed = datetime.strptime("08/1/2023 00:00",'%m/%d/%Y %H:%M')

bg_inds = time_megan.index(time_mg_bg)
ed_inds = time_megan.index(time_mg_ed)
time_plot = time_megan[bg_inds:ed_inds]


time_labels=[]
hour_mins = []
for i in  time_megan[bg_inds:ed_inds]:
    time_tmp = datetime.strptime(str(i),'%Y-%m-%d %H:%M:00')
    time_labels.append(time_tmp.strftime('%Y-%m-%d'))
    hour_mins.append(time_tmp.hour + time_tmp.minute/60.)
#time_mg_plot = datetime.fromtimestamp(time_megan[bg_inds:ed_inds])
Default_MEGAN = Default_MEGAN_all[bg_inds:ed_inds]
New_MEGAN = New_MEGAN_all[bg_inds:ed_inds]
Tem_MEGAN = Tem_MEGAN_all[bg_inds:ed_inds]
Leaf_Tem_MEGAN = Leaf_Tem_MEGAN_all[bg_inds:ed_inds]


#==================================================================

hour_mins =np.array(hour_mins)
inds_day = np.where((hour_mins >= 7) & (hour_mins <= 21))
New_MEGAN_Day = New_MEGAN[inds_day]
Default_MEGAN_Day = Default_MEGAN[inds_day]
Leaf_Tem_MEGAN_Day = Leaf_Tem_MEGAN[inds_day]
x = np.arange(5,33,1)
p0_vars = [2,0.3]
nan_inds = np.argwhere(np.logical_not(np.isnan(Leaf_Tem_MEGAN_Day)))[:,0]

pars, pcov = curve_fit(exp_fit_mg, Leaf_Tem_MEGAN_Day[nan_inds],
                       New_MEGAN_Day[nan_inds], p0=p0_vars)
mods = exp_fit_mg(Leaf_Tem_MEGAN_Day[nan_inds],pars[0],pars[1])
r2_new,res_tot = calculate_r2(New_MEGAN_Day[nan_inds], mods)
y_new_megan = exp_fit_mg(x,pars[0],pars[1])
print(r2_new,pars)
eq_new_txt = "y="+"{0:0.1f}".format(pars[0])+"$\ite$$^{0.15(T-30)}$"


pars, pcov = curve_fit(exp_fit_mg, Leaf_Tem_MEGAN_Day[nan_inds],
                       Default_MEGAN_Day[nan_inds], p0=p0_vars)
mods = exp_fit_mg(Leaf_Tem_MEGAN_Day[nan_inds],pars[0],pars[1])
r2_def,res_tot = calculate_r2(Default_MEGAN_Day[nan_inds], mods)
y_def_megan = exp_fit_mg(x,pars[0],pars[1])
print(r2_def,pars)
eq_def_txt = "y="+"{0:0.1f}".format(pars[0])+"$\ite$$^{0.13(T-30)}$"

#==================================================================

Total_New= np.nansum(New_MEGAN)
Total_Df = np.nansum(Default_MEGAN)
print("Isop increased by (%):",(np.nansum(New_MEGAN)/np.nansum(Default_MEGAN)-1)*100)
 
uniq_time_mg = np.unique(hour_mins)
avg_new_mg = np.zeros(len(uniq_time_mg))
avg_df_mg = np.zeros(len(uniq_time_mg))
std_new_mg = np.zeros(len(uniq_time_mg))
std_df_mg = np.zeros(len(uniq_time_mg))
for i in range(len(uniq_time_mg)):
    inds_mg = np.where(hour_mins == uniq_time_mg[i])
    avg_new_mg[i] = np.nanmean(New_MEGAN[inds_mg])
    avg_df_mg[i] = np.nanmean(Default_MEGAN[inds_mg])
    std_new_mg[i] = np.nanstd(New_MEGAN[inds_mg])
    std_df_mg[i] = np.nanstd(Default_MEGAN[inds_mg])
    
    


#==================================================================


fig1 = plt.figure(tight_layout=True,figsize=(16, 16 ))
gs = gridspec.GridSpec(2, 2,hspace=0.3,wspace=0.3)

ax1 = fig1.add_subplot(gs[0, :])
ax3 = fig1.add_subplot(gs[1, 0])
ax4 = fig1.add_subplot(gs[1, 1])

#fig1, ax1 = plt.subplots(nrows=1, ncols=1, figsize=(15, 7 ))
ax2 = ax1.twinx()  # instantiate a second axes that shares the same x-axis

ax1.plot(time_plot, New_MEGAN, color="darkorange",linestyle='-',linewidth=4,label="Updated MEGAN") 
ax1.plot(time_plot, Default_MEGAN, color="darkgreen",linestyle='--',linewidth=4,label="Default MEGAN") 



ax1.legend(frameon = False,loc = "upper left",fontsize=20)
ax1.set_xlabel("Date",fontsize =20)
ax1.set_ylabel("Normalized isoprene emission",fontsize=20)
ax1.tick_params(axis='both', which='major', labelsize=18)
ax1.spines.top.set_visible(False)
ax1.set_xticklabels(time_labels[::1440])
ax1.set_xticks(time_plot[::1440])#  200, 500, 700])
#ax1.tick_params( bottom = False)
ax2.set_ylabel('Air Temp. (°C)',fontsize=20,color="lightpink")  # we already handled the x-label with ax1
#ax2.plot(tem_time, tem_avg, color="darkorange",linestyle='-',linewidth=4)
ax2.plot(time_plot, Tem_MEGAN, color="lightpink",linestyle='--',linewidth=2)
ax1.set_ylim([0,16])
ax2.set_ylim([0,28])
ax2.tick_params(axis='both', which='major', labelsize=18)
ax2.spines.top.set_visible(False)
#=================figure 4====================
ax3.plot(uniq_time_mg, avg_new_mg,\
         linestyle="-", linewidth=4,color="darkorange",label="Updated MEGAN")

ax3.plot(uniq_time_mg, avg_df_mg,\
         linestyle="--", linewidth=4,color="darkgreen",label="Default MEGAN")



ax3.fill_between(uniq_time_mg,\
                 avg_new_mg-std_new_mg, avg_new_mg+std_new_mg, \
                 facecolor='sandybrown', zorder=0,alpha =0.3)
    


ax3.fill_between(uniq_time_mg,\
                 avg_df_mg-std_df_mg, avg_df_mg+std_df_mg, \
                 facecolor='lightblue', zorder=0,alpha =0.3)
ax3.spines.top.set_visible(False)
ax3.spines.right.set_visible(False)
ax3.legend(loc = "upper left",fontsize= 20,frameon = False)
ax3.tick_params(axis='both', which='major', labelsize=18)
ax3.set_xlabel("Time (hour)",fontsize =20)    
ax3.set_ylabel("Isoprene emission (nmol m$^{-2}$ s$^{-1}$)",fontsize=20)
ax3.set_xlim([0,24])
ax3.set_ylim([-0.3,10])


#============================================
ax4.plot(x, y_new_megan,\
         linestyle="-",color="darkorange",linewidth=6,\
         label="Updated MEGAN")
ax4.plot(x, y_def_megan,\
         linestyle="--",color="darkgreen",linewidth=6,
         label="Default MEGAN")    
    
ax4.plot(Leaf_Tem_MEGAN_Day, New_MEGAN_Day,\
         linestyle="", marker = "o",color="darkorange",alpha = 0.05)
ax4.plot(Leaf_Tem_MEGAN_Day, Default_MEGAN_Day,\
         linestyle="", marker = "o",color="darkgreen",alpha = 0.05)

ax4.set_xlabel("Leaf Temp. (°C)",fontsize =20)    
ax4.set_ylabel("Isoprene emission (nmol m$^{-2}$ s$^{-1}$)",fontsize=20)
ax4.set_xlim([5,35])
ax4.set_ylim([0,16])
ax4.spines.top.set_visible(False)
ax4.spines.right.set_visible(False)
ax4.legend(loc = "upper left",fontsize= 20,frameon = False)
ax4.tick_params(axis='both', which='major', labelsize=18)
ax4.text(14, 9.5, eq_new_txt, ha='center',fontsize =24,color = "darkorange")
ax4.text(13.5, 8., eq_def_txt, ha='center',fontsize =24,color = "darkgreen")



fig1.text(0.119,.895,"(a)",fontsize=22,color = "k",fontweight='bold')
fig1.text(0.119,.47,"(b)",fontsize=22,color = "k",fontweight='bold')
fig1.text(0.557,.47,"(c)",fontsize=22,color = "k",fontweight='bold')

plt.savefig("Figures/Fig3.MEGAN_Simulation.png",dpi=300,bbox_inches = 'tight')
